@@ -228,7 +228,6 @@ container.sounds.Add(SoundSet("menu", ["audio/music_menu.wv"]))
 image_null = Image("null", "")
 image_particles = Image("particles", "particles.png")
 image_game = Image("game", "game.png")
-image_browseicons = Image("browseicons", "browse_icons.png")
 image_emoticons = Image("emoticons", "emoticons.png")
 image_speedup_arrow = Image("speedup_arrow", "editor/speed_arrow.png")
 image_demobuttons = Image("demobuttons", "demo_buttons.png")
@@ -246,7 +245,6 @@ container.images.Add(image_particles)
 container.images.Add(Image("cursor", "gui_cursor.png"))
 container.images.Add(Image("banner", "gui_logo.png"))
 container.images.Add(image_emoticons)
-container.images.Add(image_browseicons)
 container.images.Add(Image("console_bg", "console.png"))
 container.images.Add(Image("console_bar", "console_bar.png"))
 container.images.Add(image_speedup_arrow)
@@ -267,7 +265,6 @@ container.pickups.Add(Pickup("ninja", 90, 90))
 set_particles = SpriteSet("particles", image_particles, 8, 8)
 set_game = SpriteSet("game", image_game, 32, 16)
 set_tee = SpriteSet("tee", image_null, 8, 4)
-set_browseicons = SpriteSet("browseicons", image_browseicons, 6, 1)
 set_emoticons = SpriteSet("emoticons", image_emoticons, 4, 4)
 set_speedup_arrow = SpriteSet("speedup_arrow", image_speedup_arrow, 1, 1)
 set_demobuttons = SpriteSet("demobuttons", image_demobuttons, 5, 1)
@@ -281,7 +278,6 @@ set_strongweak = SpriteSet("strongweak", image_strongweak, 2, 1)
 container.spritesets.Add(set_particles)
 container.spritesets.Add(set_game)
 container.spritesets.Add(set_tee)
-container.spritesets.Add(set_browseicons)
 container.spritesets.Add(set_emoticons)
 container.spritesets.Add(set_speedup_arrow)
 container.spritesets.Add(set_demobuttons)
@@ -398,12 +394,6 @@ container.sprites.Add(Sprite("wtf", set_emoticons, 1, 3, 1, 1))
 container.sprites.Add(Sprite("eyes", set_emoticons, 2, 3, 1, 1))
 container.sprites.Add(Sprite("question", set_emoticons, 3, 3, 1, 1))
 
-container.sprites.Add(Sprite("browse_lock", set_browseicons, 0,0,1,1))
-container.sprites.Add(Sprite("browse_heart", set_browseicons, 1,0,1,1))
-container.sprites.Add(Sprite("browse_unpure", set_browseicons, 3,0,1,1))
-container.sprites.Add(Sprite("browse_ddnet", set_browseicons, 4,0,1,1))
-container.sprites.Add(Sprite("browse_rank", set_browseicons, 5,0,1,1))
-
 container.sprites.Add(Sprite("speedup_arrow", set_speedup_arrow, 0,0,1,1))
 
 container.sprites.Add(Sprite("demobutton_play", set_demobuttons, 0,0,1,1))
