@@ -96,7 +96,7 @@ public:
 		z = ((col >> 0) & 0xFF) / 255.0f;
 	}
 
-	vec4 v4() const { return vec4(x, y, z, a); };
+	vec4 v4() const { return vec4(x, y, z, a); }
 
 	unsigned Pack(bool Alpha = true)
 	{
