@@ -5,6 +5,7 @@
 
 #include <base/detect.h>
 
+#include "engine/graphics.h"
 #include "graphics_defines.h"
 
 #include "blocklist_driver.h"
@@ -58,6 +59,7 @@ public:
 	CGraphicsBackend_Threaded();
 
 	virtual void RunBuffer(CCommandBuffer *pBuffer);
+	virtual void RunBufferSingleThreadedUnsafe(CCommandBuffer *pBuffer);
 	virtual bool IsIdle() const;
 	virtual void WaitForIdle();
 
@@ -69,9 +71,9 @@ private:
 	ICommandProcessor *m_pProcessor;
 	std::mutex m_BufferSwapMutex;
 	std::condition_variable m_BufferSwapCond;
-	std::condition_variable m_BufferDoneCond;
 	CCommandBuffer *m_pBuffer;
 	std::atomic_bool m_Shutdown;
+	bool m_Started = false;
 	std::atomic_bool m_BufferInProcess;
 	void *m_Thread;
 
@@ -88,12 +90,6 @@ public:
 	bool RunCommand(const CCommandBuffer::SCommand *pBaseCommand);
 };
 
-enum EBackendType
-{
-	BACKEND_TYPE_OPENGL = 0,
-	BACKEND_TYPE_OPENGL_ES,
-};
-
 struct SBackendCapabilites
 {
 	bool m_TileBuffering;
@@ -108,6 +104,9 @@ struct SBackendCapabilites
 	bool m_2DArrayTexturesAsExtension;
 	bool m_ShaderSupport;
 
+	// use quads as much as possible, even if the user config says otherwise
+	bool m_TrianglesAsQuads;
+
 	int m_ContextMajor;
 	int m_ContextMinor;
 	int m_ContextPatch;
@@ -155,75 +154,38 @@ public:
 	bool RunCommand(const CCommandBuffer::SCommand *pBaseCommand);
 };
 
-class CCommandProcessorFragment_OpenGLBase
-{
-public:
-	virtual ~CCommandProcessorFragment_OpenGLBase() = default;
-	virtual bool RunCommand(const CCommandBuffer::SCommand *pBaseCommand) = 0;
-
-	enum
-	{
-		CMD_INIT = CCommandBuffer::CMDGROUP_PLATFORM_OPENGL,
-		CMD_SHUTDOWN = CMD_INIT + 1,
-	};
-
-	struct SCommand_Init : public CCommandBuffer::SCommand
-	{
-		SCommand_Init() :
-			SCommand(CMD_INIT) {}
-		class IStorage *m_pStorage;
-		std::atomic<int> *m_pTextureMemoryUsage;
-		SBackendCapabilites *m_pCapabilities;
-		int *m_pInitError;
-
-		const char **m_pErrStringPtr;
-
-		char *m_pVendorString;
-		char *m_pVersionString;
-		char *m_pRendererString;
-
-		int m_RequestedMajor;
-		int m_RequestedMinor;
-		int m_RequestedPatch;
-
-		EBackendType m_RequestedBackend;
-
-		int m_GlewMajor;
-		int m_GlewMinor;
-		int m_GlewPatch;
-	};
-
-	struct SCommand_Shutdown : public CCommandBuffer::SCommand
-	{
-		SCommand_Shutdown() :
-			SCommand(CMD_SHUTDOWN) {}
-	};
-};
-
 // command processor impelementation, uses the fragments to combine into one processor
-class CCommandProcessor_SDL_OpenGL : public CGraphicsBackend_Threaded::ICommandProcessor
+class CCommandProcessor_SDL_GL : public CGraphicsBackend_Threaded::ICommandProcessor
 {
-	CCommandProcessorFragment_OpenGLBase *m_pOpenGL;
+	class CCommandProcessorFragment_GLBase *m_pGLBackend;
 	CCommandProcessorFragment_SDL m_SDL;
 	CCommandProcessorFragment_General m_General;
 
 	EBackendType m_BackendType;
 
 public:
-	CCommandProcessor_SDL_OpenGL(EBackendType BackendType, int OpenGLMajor, int OpenGLMinor, int OpenGLPatch);
-	virtual ~CCommandProcessor_SDL_OpenGL();
+	CCommandProcessor_SDL_GL(EBackendType BackendType, int GLMajor, int GLMinor, int GLPatch);
+	virtual ~CCommandProcessor_SDL_GL();
 	virtual void RunBuffer(CCommandBuffer *pBuffer);
 };
 
 static constexpr size_t gs_GPUInfoStringSize = 256;
 
-// graphics backend implemented with SDL and OpenGL
-class CGraphicsBackend_SDL_OpenGL : public CGraphicsBackend_Threaded
+// graphics backend implemented with SDL and the graphics library @see EBackendType
+class CGraphicsBackend_SDL_GL : public CGraphicsBackend_Threaded
 {
 	SDL_Window *m_pWindow = NULL;
 	SDL_GLContext m_GLContext;
-	ICommandProcessor *m_pProcessor;
-	std::atomic<int> m_TextureMemoryUsage;
+	ICommandProcessor *m_pProcessor = nullptr;
+	std::atomic<uint64_t> m_TextureMemoryUsage{0};
+	std::atomic<uint64_t> m_BufferMemoryUsage{0};
+	std::atomic<uint64_t> m_StreamMemoryUsage{0};
+	std::atomic<uint64_t> m_StagingMemoryUsage{0};
+
+	TTWGraphicsGPUList m_GPUList;
+
+	TGLBackendReadPresentedImageData m_ReadPresentedImageDataFunc;
+
 	int m_NumScreens;
 
 	SBackendCapabilites m_Capabilites;
@@ -232,8 +194,7 @@ class CGraphicsBackend_SDL_OpenGL : public CGraphicsBackend_Threaded
 	char m_aVersionString[gs_GPUInfoStringSize] = {};
 	char m_aRendererString[gs_GPUInfoStringSize] = {};
 
-	bool m_UseNewOpenGL;
-	EBackendType m_BackendType;
+	EBackendType m_BackendType = BACKEND_TYPE_AUTO;
 
 	char m_aErrorString[256];
 
@@ -241,11 +202,16 @@ class CGraphicsBackend_SDL_OpenGL : public CGraphicsBackend_Threaded
 	static void ClampDriverVersion(EBackendType BackendType);
 
 public:
-	CGraphicsBackend_SDL_OpenGL();
+	CGraphicsBackend_SDL_GL();
 	virtual int Init(const char *pName, int *Screen, int *pWidth, int *pHeight, int *pRefreshRate, int FsaaSamples, int Flags, int *pDesktopWidth, int *pDesktopHeight, int *pCurrentWidth, int *pCurrentHeight, class IStorage *pStorage);
 	virtual int Shutdown();
 
-	virtual int MemoryUsage() const;
+	virtual uint64_t TextureMemoryUsage() const;
+	virtual uint64_t BufferMemoryUsage() const;
+	virtual uint64_t StreamedMemoryUsage() const;
+	virtual uint64_t StagingMemoryUsage() const;
+
+	virtual const TTWGraphicsGPUList &GetGPUs() const;
 
 	virtual int GetNumScreens() const { return m_NumScreens; }
 
@@ -268,9 +234,9 @@ public:
 	virtual void WindowDestroyNtf(uint32_t WindowID);
 	virtual void WindowCreateNtf(uint32_t WindowID);
 
-	virtual void GetDriverVersion(EGraphicsDriverAgeType DriverAgeType, int &Major, int &Minor, int &Patch);
+	virtual bool GetDriverVersion(EGraphicsDriverAgeType DriverAgeType, int &Major, int &Minor, int &Patch, const char *&pName, EBackendType BackendType);
 	virtual bool IsConfigModernAPI() { return IsModernAPI(m_BackendType); }
-	virtual bool IsNewOpenGL() { return m_UseNewOpenGL; }
+	virtual bool UseTrianglesAsQuad() { return m_Capabilites.m_TrianglesAsQuads; }
 	virtual bool HasTileBuffering() { return m_Capabilites.m_TileBuffering; }
 	virtual bool HasQuadBuffering() { return m_Capabilites.m_QuadBuffering; }
 	virtual bool HasTextBuffering() { return m_Capabilites.m_TextBuffering; }
@@ -300,6 +266,8 @@ public:
 		return m_aRendererString;
 	}
 
+	virtual TGLBackendReadPresentedImageData &GetReadPresentedImageDataFuncUnsafe();
+
 	static bool IsModernAPI(EBackendType BackendType);
 };
 
