@@ -6,7 +6,7 @@
 #include "serverbrowser_ping_cache.h"
 
 #include <algorithm>
-#include <limits.h>
+#include <climits>
 
 #include <base/hash_ctxt.h>
 #include <base/math.h>
@@ -30,6 +30,8 @@
 
 #include <engine/external/json-parser/json.h>
 
+#include <game/client/components/menus.h> // PAGE_DDNET
+
 class SortWrap
 {
 	typedef bool (CServerBrowser::*SortFunc)(int, int) const;
@@ -75,14 +77,9 @@ CServerBrowser::CServerBrowser()
 
 CServerBrowser::~CServerBrowser()
 {
-	if(m_ppServerlist)
-		free(m_ppServerlist);
-
-	if(m_pSortedServerlist)
-		free(m_pSortedServerlist);
-
-	if(m_pDDNetInfo)
-		json_value_free(m_pDDNetInfo);
+	free(m_ppServerlist);
+	free(m_pSortedServerlist);
+	json_value_free(m_pDDNetInfo);
 
 	delete m_pHttp;
 	m_pHttp = nullptr;
@@ -265,8 +262,7 @@ void CServerBrowser::Filter()
 	// allocate the sorted list
 	if(m_NumSortedServersCapacity < m_NumServers)
 	{
-		if(m_pSortedServerlist)
-			free(m_pSortedServerlist);
+		free(m_pSortedServerlist);
 		m_NumSortedServersCapacity = m_NumServers;
 		m_pSortedServerlist = (int *)calloc(m_NumSortedServersCapacity, sizeof(int));
 	}
@@ -684,10 +680,10 @@ void CServerBrowser::Set(const NETADDR &Addr, int Type, int Token, const CServer
 			NETADDR Broadcast;
 			mem_zero(&Broadcast, sizeof(Broadcast));
 			Broadcast.type = m_pNetClient->NetType() | NETTYPE_LINK_BROADCAST;
-			int Token = GenerateToken(Broadcast);
+			int TokenBC = GenerateToken(Broadcast);
 			bool Drop = false;
-			Drop = Drop || BasicToken != GetBasicToken(Token);
-			Drop = Drop || (pInfo->m_Type == SERVERINFO_EXTENDED && ExtraToken != GetExtraToken(Token));
+			Drop = Drop || BasicToken != GetBasicToken(TokenBC);
+			Drop = Drop || (pInfo->m_Type == SERVERINFO_EXTENDED && ExtraToken != GetExtraToken(TokenBC));
 			if(Drop)
 			{
 				return;
@@ -702,10 +698,10 @@ void CServerBrowser::Set(const NETADDR &Addr, int Type, int Token, const CServer
 			{
 				return;
 			}
-			int Token = GenerateToken(Addr);
+			int TokenAddr = GenerateToken(Addr);
 			bool Drop = false;
-			Drop = Drop || BasicToken != GetBasicToken(Token);
-			Drop = Drop || (pInfo->m_Type == SERVERINFO_EXTENDED && ExtraToken != GetExtraToken(Token));
+			Drop = Drop || BasicToken != GetBasicToken(TokenAddr);
+			Drop = Drop || (pInfo->m_Type == SERVERINFO_EXTENDED && ExtraToken != GetExtraToken(TokenAddr));
 			if(Drop)
 			{
 				return;
@@ -1194,7 +1190,7 @@ void CServerBrowser::Update(bool ForceResort)
 
 	CServerEntry *pEntry = m_pFirstReqServer;
 	int Count = 0;
-	while(1)
+	while(true)
 	{
 		if(!pEntry) // no more entries
 			break;
@@ -1223,7 +1219,7 @@ void CServerBrowser::Update(bool ForceResort)
 	{
 		//reset old ones
 		pEntry = m_pFirstReqServer;
-		while(1)
+		while(true)
 		{
 			if(!pEntry) // no more entries
 				break;
@@ -1239,7 +1235,7 @@ void CServerBrowser::Update(bool ForceResort)
 	else if(Count == 0 && m_CurrentMaxRequests == 1) //we reached the limit, just release all left requests. IF a server sends us a packet, a new request will be added automatically, so we can delete all
 	{
 		pEntry = m_pFirstReqServer;
-		while(1)
+		while(true)
 		{
 			if(!pEntry) // no more entries
 				break;
@@ -1501,8 +1497,7 @@ void CServerBrowser::LoadDDNetInfoJson()
 	io_read(File, pBuf, Length);
 	io_close(File);
 
-	if(m_pDDNetInfo)
-		json_value_free(m_pDDNetInfo);
+	json_value_free(m_pDDNetInfo);
 
 	m_pDDNetInfo = json_parse(pBuf, Length);
 
@@ -1527,6 +1522,38 @@ void CServerBrowser::LoadDDNetInfoJson()
 	}
 }
 
+const char *CServerBrowser::GetTutorialServer()
+{
+	// Use DDNet tab as default after joining tutorial, also makes sure Find() actually works
+	// Note that when no server info has been loaded yet, this will not return a result immediately.
+	g_Config.m_UiPage = CMenus::PAGE_DDNET;
+	Refresh(IServerBrowser::TYPE_DDNET);
+
+	CNetwork *pNetwork = &m_aNetworks[NETWORK_DDNET];
+	const char *pBestAddr = nullptr;
+	int BestLatency = std::numeric_limits<int>::max();
+
+	for(int i = 0; i < pNetwork->m_NumCountries; i++)
+	{
+		CNetworkCountry *pCntr = &pNetwork->m_aCountries[i];
+		for(int j = 0; j < pCntr->m_NumServers; j++)
+		{
+			CServerEntry *pEntry = Find(pCntr->m_aServers[j]);
+			if(!pEntry)
+				continue;
+			if(str_find(pEntry->m_Info.m_aName, "(Tutorial)") == 0)
+				continue;
+			if(pEntry->m_Info.m_NumPlayers > pEntry->m_Info.m_MaxPlayers - 10)
+				continue;
+			if(pEntry->m_Info.m_Latency >= BestLatency)
+				continue;
+			BestLatency = pEntry->m_Info.m_Latency;
+			pBestAddr = pEntry->m_Info.m_aAddress;
+		}
+	}
+	return pBestAddr;
+}
+
 const json_value *CServerBrowser::LoadDDNetInfo()
 {
 	LoadDDNetInfoJson();
@@ -1626,11 +1653,11 @@ void CServerBrowser::CountryFilterClean(int Network)
 	char aNewList[128];
 	aNewList[0] = '\0';
 
-	for(auto &Network : m_aNetworks)
+	for(auto &Net : m_aNetworks)
 	{
-		for(int i = 0; i < Network.m_NumCountries; i++)
+		for(int i = 0; i < Net.m_NumCountries; i++)
 		{
-			const char *pName = Network.m_aCountries[i].m_aName;
+			const char *pName = Net.m_aCountries[i].m_aName;
 			if(DDNetFiltered(pExcludeCountries, pName))
 			{
 				char aBuf[128];
@@ -1743,7 +1770,7 @@ bool IsBlockInfectionZ(const CServerInfo *pInfo)
 
 bool IsBlockWorlds(const CServerInfo *pInfo)
 {
-	return (str_comp_nocase_num(pInfo->m_aGameType, "bw  ", 4) == 0) || (str_comp_nocase(pInfo->m_aGameType, "bw") == 0);
+	return (str_startswith(pInfo->m_aGameType, "bw  ")) || (str_comp_nocase(pInfo->m_aGameType, "bw") == 0);
 }
 
 bool IsCity(const CServerInfo *pInfo)
