@@ -110,18 +110,19 @@ public:
 	void FavoriteAllowPing(const NETADDR &Addr, bool AllowPing);
 	void RemoveFavorite(const NETADDR &Addr);
 
+	virtual const char *GetTutorialServer();
 	void LoadDDNetRanks();
 	void RecheckOfficial();
 	void LoadDDNetServers();
 	void LoadDDNetInfoJson();
 	const json_value *LoadDDNetInfo();
 	int HasRank(const char *pMap);
-	int NumCountries(int Network) { return m_aNetworks[Network].m_NumCountries; };
-	int GetCountryFlag(int Network, int Index) { return m_aNetworks[Network].m_aCountries[Index].m_FlagID; };
-	const char *GetCountryName(int Network, int Index) { return m_aNetworks[Network].m_aCountries[Index].m_aName; };
+	int NumCountries(int Network) { return m_aNetworks[Network].m_NumCountries; }
+	int GetCountryFlag(int Network, int Index) { return m_aNetworks[Network].m_aCountries[Index].m_FlagID; }
+	const char *GetCountryName(int Network, int Index) { return m_aNetworks[Network].m_aCountries[Index].m_aName; }
 
-	int NumTypes(int Network) { return m_aNetworks[Network].m_NumTypes; };
-	const char *GetType(int Network, int Index) { return m_aNetworks[Network].m_aTypes[Index]; };
+	int NumTypes(int Network) { return m_aNetworks[Network].m_NumTypes; }
+	const char *GetType(int Network, int Index) { return m_aNetworks[Network].m_aTypes[Index]; }
 
 	void DDNetFilterAdd(char *pFilter, const char *pName);
 	void DDNetFilterRem(char *pFilter, const char *pName);
@@ -142,7 +143,7 @@ public:
 	void RequestImpl64(const NETADDR &Addr, CServerEntry *pEntry) const;
 	void QueueRequest(CServerEntry *pEntry);
 	CServerEntry *Find(const NETADDR &Addr);
-	int GetCurrentType() { return m_ServerlistType; };
+	int GetCurrentType() { return m_ServerlistType; }
 
 private:
 	CNetClient *m_pNetClient;
