@@ -14,9 +14,16 @@
 #include <game/generated/protocol.h>
 #include <game/generated/protocol7.h>
 #include <game/generated/protocolglue.h>
+#include <game/version.h>
 
 struct CAntibotRoundData;
 
+// When recording a demo on the server, the ClientID -1 is used
+enum
+{
+	SERVER_DEMO_CLIENT = -1
+};
+
 class IServer : public IInterface
 {
 	MACRO_INTERFACE("server", 0)
@@ -64,7 +71,7 @@ public:
 		T tmp;
 		if(ClientID == -1)
 		{
-			for(int i = 0; i < MAX_CLIENTS; i++)
+			for(int i = 0; i < MaxClients(); i++)
 				if(ClientIngame(i))
 				{
 					mem_copy(&tmp, pMsg, sizeof(T));
@@ -85,7 +92,7 @@ public:
 		int Result = 0;
 		if(ClientID == -1)
 		{
-			for(int i = 0; i < MAX_CLIENTS; i++)
+			for(int i = 0; i < MaxClients(); i++)
 				if(ClientIngame(i) && IsSixup(i))
 					Result = SendPackMsgOne(pMsg, Flags, i);
 		}
@@ -150,13 +157,19 @@ public:
 		return SendMsg(&Packer, Flags, ClientID);
 	}
 
+	int GetClientVersion(int ClientID) const
+	{
+		CClientInfo Info;
+		GetClientInfo(ClientID, &Info);
+		return Info.m_DDNetVersion;
+	}
+
 	bool Translate(int &Target, int Client)
 	{
 		if(IsSixup(Client))
 			return true;
-		CClientInfo Info;
-		GetClientInfo(Client, &Info);
-		if(Info.m_DDNetVersion >= VERSION_DDNET_OLD)
+		int ClientVersion = Client != SERVER_DEMO_CLIENT ? GetClientVersion(Client) : CLIENT_VERSIONNR;
+		if(ClientVersion >= VERSION_DDNET_OLD)
 			return true;
 		int *pMap = GetIdMap(Client);
 		bool Found = false;
@@ -176,9 +189,8 @@ public:
 	{
 		if(IsSixup(Client))
 			return true;
-		CClientInfo Info;
-		GetClientInfo(Client, &Info);
-		if(Info.m_DDNetVersion >= VERSION_DDNET_OLD)
+		int ClientVersion = Client != SERVER_DEMO_CLIENT ? GetClientVersion(Client) : CLIENT_VERSIONNR;
+		if(ClientVersion >= VERSION_DDNET_OLD)
 			return true;
 		Target = clamp(Target, 0, VANILLA_MAX_CLIENTS - 1);
 		int *pMap = GetIdMap(Client);
