@@ -70,8 +70,8 @@ public:
 	virtual void ToUnixTimestamp(const char *pTimestamp, char *aBuf, unsigned int BufferSize);
 	virtual const char *InsertTimestampAsUtc() const { return "?"; }
 	virtual const char *CollateNocase() const { return "CONVERT(? USING utf8mb4) COLLATE utf8mb4_general_ci"; }
-	virtual const char *InsertIgnore() const { return "INSERT IGNORE"; };
-	virtual const char *Random() const { return "RAND()"; };
+	virtual const char *InsertIgnore() const { return "INSERT IGNORE"; }
+	virtual const char *Random() const { return "RAND()"; }
 	virtual const char *MedianMapTime(char *pBuffer, int BufferSize) const;
 	virtual const char *False() const { return "FALSE"; }
 	virtual const char *True() const { return "TRUE"; }
