@@ -18,9 +18,7 @@ IJob &IJob::operator=(const IJob &Other)
 	return *this;
 }
 
-IJob::~IJob()
-{
-}
+IJob::~IJob() = default;
 
 int IJob::Status()
 {
@@ -40,16 +38,10 @@ CJobPool::CJobPool()
 
 CJobPool::~CJobPool()
 {
-	m_Shutdown = true;
-	for(int i = 0; i < m_NumThreads; i++)
-		sphore_signal(&m_Semaphore);
-	for(int i = 0; i < m_NumThreads; i++)
+	if(!m_Shutdown)
 	{
-		if(m_apThreads[i])
-			thread_wait(m_apThreads[i]);
+		Destroy();
 	}
-	lock_destroy(m_Lock);
-	sphore_destroy(&m_Semaphore);
 }
 
 void CJobPool::WorkerThread(void *pUser)
@@ -88,6 +80,20 @@ void CJobPool::Init(int NumThreads)
 		m_apThreads[i] = thread_init(WorkerThread, this, "CJobPool worker");
 }
 
+void CJobPool::Destroy()
+{
+	m_Shutdown = true;
+	for(int i = 0; i < m_NumThreads; i++)
+		sphore_signal(&m_Semaphore);
+	for(int i = 0; i < m_NumThreads; i++)
+	{
+		if(m_apThreads[i])
+			thread_wait(m_apThreads[i]);
+	}
+	lock_destroy(m_Lock);
+	sphore_destroy(&m_Semaphore);
+}
+
 void CJobPool::Add(std::shared_ptr<IJob> pJob)
 {
 	lock_wait(m_Lock);
