@@ -1,6 +1,6 @@
 /* (c) Magnus Auvinen. See licence.txt in the root of the distribution for more information. */
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
-#include <stdio.h> // sscanf
+#include <cstdio> // sscanf
 
 #include <base/system.h>
 
@@ -155,7 +155,7 @@ public:
 
 		CLineReader LineReader;
 		LineReader.Init(File);
-		while(1)
+		while(true)
 		{
 			CMasterInfo Info = {{0}};
 			const char *pLine = LineReader.Get();
