@@ -7,18 +7,15 @@
 #include "netban.h"
 #include "network.h"
 
-bool CNetConsole::Open(NETADDR BindAddr, CNetBan *pNetBan, int Flags)
+bool CNetConsole::Open(NETADDR BindAddr, CNetBan *pNetBan)
 {
 	// zero out the whole structure
 	mem_zero(this, sizeof(*this));
-	m_Socket.type = NETTYPE_INVALID;
-	m_Socket.ipv4sock = -1;
-	m_Socket.ipv6sock = -1;
 	m_pNetBan = pNetBan;
 
 	// open socket
 	m_Socket = net_tcp_create(BindAddr);
-	if(!m_Socket.type)
+	if(!m_Socket)
 		return false;
 	if(net_tcp_listen(m_Socket, NET_MAX_CONSOLE_CLIENTS))
 		return false;
