@@ -37,9 +37,9 @@ public:
 	enum
 	{
 		MODIFIER_NONE = 0,
-		MODIFIER_SHIFT,
 		MODIFIER_CTRL,
 		MODIFIER_ALT,
+		MODIFIER_SHIFT,
 		MODIFIER_GUI,
 		MODIFIER_COUNT,
 		MODIFIER_COMBINATION_COUNT = 1 << MODIFIER_COUNT
@@ -47,17 +47,16 @@ public:
 
 	CBindsSpecial m_SpecialBinds;
 
-	void Bind(int KeyID, const char *pStr, bool FreeOnly = false, int Modifier = MODIFIER_NONE);
+	void Bind(int KeyID, const char *pStr, bool FreeOnly = false, int ModifierCombination = MODIFIER_NONE);
 	void SetDefaults();
 	void UnbindAll();
-	const char *Get(int KeyID, int Modifier);
+	const char *Get(int KeyID, int ModifierCombination);
 	void GetKey(const char *pBindStr, char *aBuf, unsigned BufSize);
-	int GetBindSlot(const char *pBindString, int *Modifier);
+	int GetBindSlot(const char *pBindString, int *pModifierCombination);
 	static int GetModifierMask(IInput *i);
 	static int GetModifierMaskOfKey(int Key);
-	static bool ModifierMatchesKey(int Modifier, int Key);
 	static const char *GetModifierName(int Modifier);
-	static const char *GetKeyBindModifiersName(int Modifier);
+	static const char *GetKeyBindModifiersName(int ModifierCombination);
 
 	virtual void OnConsoleInit() override;
 	virtual bool OnInput(IInput::CEvent Event) override;
