@@ -339,12 +339,12 @@ bool AddTile(std::vector<SGraphicTile> &TmpTiles, std::vector<SGraphicTileTexure
 {
 	if(Index)
 	{
-		TmpTiles.push_back(SGraphicTile());
+		TmpTiles.emplace_back();
 		SGraphicTile &Tile = TmpTiles.back();
 		SGraphicTileTexureCoords *pTileTex = NULL;
 		if(DoTextureCoords)
 		{
-			TmpTileTexCoords.push_back(SGraphicTileTexureCoords());
+			TmpTileTexCoords.emplace_back();
 			SGraphicTileTexureCoords &TileTex = TmpTileTexCoords.back();
 			pTileTex = &TileTex;
 		}
@@ -839,29 +839,28 @@ void CMapLayers::OnMapLoad()
 							}
 
 							// first create the buffer object
-							int BufferObjectIndex = Graphics()->CreateBufferObject(UploadDataSize, pUploadData, true);
+							int BufferObjectIndex = Graphics()->CreateBufferObject(UploadDataSize, pUploadData, 0, true);
 
 							// then create the buffer container
 							SBufferContainerInfo ContainerInfo;
 							ContainerInfo.m_Stride = (DoTextureCoords ? (sizeof(float) * 2 + sizeof(vec3)) : 0);
-							ContainerInfo.m_Attributes.push_back(SBufferContainerInfo::SAttribute());
+							ContainerInfo.m_VertBufferBindingIndex = BufferObjectIndex;
+							ContainerInfo.m_Attributes.emplace_back();
 							SBufferContainerInfo::SAttribute *pAttr = &ContainerInfo.m_Attributes.back();
 							pAttr->m_DataTypeCount = 2;
 							pAttr->m_Type = GRAPHICS_TYPE_FLOAT;
 							pAttr->m_Normalized = false;
 							pAttr->m_pOffset = 0;
 							pAttr->m_FuncType = 0;
-							pAttr->m_VertBufferBindingIndex = BufferObjectIndex;
 							if(DoTextureCoords)
 							{
-								ContainerInfo.m_Attributes.push_back(SBufferContainerInfo::SAttribute());
+								ContainerInfo.m_Attributes.emplace_back();
 								pAttr = &ContainerInfo.m_Attributes.back();
 								pAttr->m_DataTypeCount = 3;
 								pAttr->m_Type = GRAPHICS_TYPE_FLOAT;
 								pAttr->m_Normalized = false;
 								pAttr->m_pOffset = (void *)(sizeof(vec2));
 								pAttr->m_FuncType = 0;
-								pAttr->m_VertBufferBindingIndex = BufferObjectIndex;
 							}
 
 							Visuals.m_BufferContainerIndex = Graphics()->CreateBufferContainer(&ContainerInfo);
@@ -944,36 +943,34 @@ void CMapLayers::OnMapLoad()
 					else
 						pUploadData = &tmpQuads[0];
 					// create the buffer object
-					int BufferObjectIndex = Graphics()->CreateBufferObject(UploadDataSize, pUploadData);
+					int BufferObjectIndex = Graphics()->CreateBufferObject(UploadDataSize, pUploadData, 0);
 					// then create the buffer container
 					SBufferContainerInfo ContainerInfo;
 					ContainerInfo.m_Stride = (Textured ? (sizeof(STmpQuadTextured) / 4) : (sizeof(STmpQuad) / 4));
-					ContainerInfo.m_Attributes.push_back(SBufferContainerInfo::SAttribute());
+					ContainerInfo.m_VertBufferBindingIndex = BufferObjectIndex;
+					ContainerInfo.m_Attributes.emplace_back();
 					SBufferContainerInfo::SAttribute *pAttr = &ContainerInfo.m_Attributes.back();
 					pAttr->m_DataTypeCount = 4;
 					pAttr->m_Type = GRAPHICS_TYPE_FLOAT;
 					pAttr->m_Normalized = false;
 					pAttr->m_pOffset = 0;
 					pAttr->m_FuncType = 0;
-					pAttr->m_VertBufferBindingIndex = BufferObjectIndex;
-					ContainerInfo.m_Attributes.push_back(SBufferContainerInfo::SAttribute());
+					ContainerInfo.m_Attributes.emplace_back();
 					pAttr = &ContainerInfo.m_Attributes.back();
 					pAttr->m_DataTypeCount = 4;
 					pAttr->m_Type = GRAPHICS_TYPE_UNSIGNED_BYTE;
 					pAttr->m_Normalized = true;
 					pAttr->m_pOffset = (void *)(sizeof(float) * 4);
 					pAttr->m_FuncType = 0;
-					pAttr->m_VertBufferBindingIndex = BufferObjectIndex;
 					if(Textured)
 					{
-						ContainerInfo.m_Attributes.push_back(SBufferContainerInfo::SAttribute());
+						ContainerInfo.m_Attributes.emplace_back();
 						pAttr = &ContainerInfo.m_Attributes.back();
 						pAttr->m_DataTypeCount = 2;
 						pAttr->m_Type = GRAPHICS_TYPE_FLOAT;
 						pAttr->m_Normalized = false;
 						pAttr->m_pOffset = (void *)(sizeof(float) * 4 + sizeof(unsigned char) * 4);
 						pAttr->m_FuncType = 0;
-						pAttr->m_VertBufferBindingIndex = BufferObjectIndex;
 					}
 
 					pQLayerVisuals->m_BufferContainerIndex = Graphics()->CreateBufferContainer(&ContainerInfo);
@@ -1406,6 +1403,17 @@ void CMapLayers::RenderQuadLayer(int LayerIndex, CMapItemLayerQuads *pQuadLayer,
 			Rot = aChannels[2] / 180.0f * pi;
 		}
 
+		bool NeedsFlush = QuadsRenderCount == gs_GraphicsMaxQuadsRenderCount || !(aColor[3] > 0);
+
+		if(NeedsFlush)
+		{
+			// render quads of the current offset directly(cancel batching)
+			Graphics()->RenderQuadLayer(Visuals.m_BufferContainerIndex, &s_QuadRenderInfo[0], QuadsRenderCount, CurQuadOffset);
+			QuadsRenderCount = 0;
+			// since this quad is ignored, the offset is the next quad
+			CurQuadOffset = i + 1;
+		}
+
 		if(aColor[3] > 0)
 		{
 			SQuadRenderInfo &QInfo = s_QuadRenderInfo[QuadsRenderCount++];
@@ -1414,14 +1422,6 @@ void CMapLayers::RenderQuadLayer(int LayerIndex, CMapItemLayerQuads *pQuadLayer,
 			QInfo.m_aOffsets[1] = OffsetY;
 			QInfo.m_Rotation = Rot;
 		}
-		else
-		{
-			// render quads of the current offset directly(cancel batching)
-			Graphics()->RenderQuadLayer(Visuals.m_BufferContainerIndex, &s_QuadRenderInfo[0], QuadsRenderCount, CurQuadOffset);
-			QuadsRenderCount = 0;
-			// since this quad is ignored, the offset is the next quad
-			CurQuadOffset = i + 1;
-		}
 	}
 	Graphics()->RenderQuadLayer(Visuals.m_BufferContainerIndex, &s_QuadRenderInfo[0], QuadsRenderCount, CurQuadOffset);
 }
