@@ -50,7 +50,7 @@ public:
 	bool m_TakeKey;
 	bool m_GotKey;
 	IInput::CEvent m_Key;
-	int m_Modifier;
+	int m_ModifierCombination;
 	CMenusKeyBinder();
 	virtual int Sizeof() const override { return sizeof(*this); }
 	virtual bool OnInput(IInput::CEvent Event) override;
@@ -95,7 +95,7 @@ class CMenus : public CComponent
 	int DoButton_GridHeader(const void *pID, const char *pText, int Checked, const CUIRect *pRect);
 
 	void DoButton_KeySelect(const void *pID, const char *pText, int Checked, const CUIRect *pRect);
-	int DoKeyReader(void *pID, const CUIRect *pRect, int Key, int Modifier, int *NewModifier);
+	int DoKeyReader(void *pID, const CUIRect *pRect, int Key, int ModifierCombination, int *NewModifierCombination);
 
 	void UiDoGetButtons(int Start, int Stop, CUIRect View, CUIRect ScopeView);
 
@@ -166,7 +166,9 @@ class CMenus : public CComponent
 						if(pText == NULL)
 							pText = GetTextLambda();
 						NewRect.m_Text = pText;
-						UI()->DoLabel(NewRect, &Text, pText, Text.h * CUI::ms_FontmodHeight, TEXTALIGN_CENTER, -1, AlignVertically);
+						SLabelProperties Props;
+						Props.m_AlignVertically = AlignVertically;
+						UI()->DoLabel(NewRect, &Text, pText, Text.h * CUI::ms_FontmodHeight, TEXTALIGN_CENTER, Props);
 					}
 				}
 				Graphics()->SetColor(1, 1, 1, 1);
@@ -241,6 +243,8 @@ protected:
 	sorted_array<SCustomEmoticon> m_EmoticonList;
 	sorted_array<SCustomParticle> m_ParticlesList;
 
+	bool m_IsInit = false;
+
 	static void LoadEntities(struct SCustomEntities *pEntitiesItem, void *pUser);
 	static int EntitiesScan(const char *pName, int IsDir, int DirType, void *pUser);
 
@@ -263,6 +267,7 @@ protected:
 	bool m_MenuActive;
 	bool m_UseMouseButtons;
 	vec2 m_MousePos;
+	bool m_JoinTutorial;
 
 	char m_aNextServer[256];
 
@@ -414,6 +419,8 @@ protected:
 	int m_DemolistStorageType;
 	int m_Speed = 4;
 
+	int64_t m_DemoPopulateStartTime = 0;
+
 	void DemolistOnUpdate(bool Reset);
 	//void DemolistPopulate();
 	static int DemolistFetchCallback(const CFsFileInfo *pInfo, int IsDir, int StorageType, void *pUser);
@@ -517,8 +524,9 @@ public:
 	CMenus();
 	virtual int Sizeof() const override { return sizeof(*this); }
 
-	void RenderLoading();
-	void RenderUpdating(const char *pCaption, int current = 0, int total = 0);
+	void RenderLoading(bool IncreaseCounter, bool RenderLoadingBar = true);
+
+	bool IsInit() { return m_IsInit; }
 
 	bool IsActive() const { return m_MenuActive; }
 	void KillServer();
@@ -618,6 +626,8 @@ public:
 
 	sorted_array<CGhostItem> m_lGhosts;
 
+	int64_t m_GhostPopulateStartTime = 0;
+
 	void GhostlistPopulate();
 	CGhostItem *GetOwnGhost();
 	void UpdateOwnGhost(CGhostItem Item);
