@@ -1,6 +1,6 @@
 /* (c) Redix and Sushi */
 
-#include <ctype.h>
+#include <cctype>
 
 #include <base/system.h>
 #include <engine/serverbrowser.h>
@@ -180,9 +180,16 @@ void CRaceDemo::StopRecord(int Time)
 	m_RecordStopTick = -1;
 }
 
+struct SRaceDemoFetchUser
+{
+	CRaceDemo *m_pThis;
+	CDemoListParam *m_pParam;
+};
+
 int CRaceDemo::RaceDemolistFetchCallback(const CFsFileInfo *pInfo, int IsDir, int StorageType, void *pUser)
 {
-	CDemoListParam *pParam = (CDemoListParam *)pUser;
+	auto *pRealUser = (SRaceDemoFetchUser *)pUser;
+	auto *pParam = pRealUser->m_pParam;
 	int MapLen = str_length(pParam->pMap);
 	if(IsDir || !str_endswith(pInfo->m_pName, ".demo") || !str_startswith(pInfo->m_pName, pParam->pMap) || pInfo->m_pName[MapLen] != '_')
 		return 0;
@@ -211,14 +218,23 @@ int CRaceDemo::RaceDemolistFetchCallback(const CFsFileInfo *pInfo, int IsDir, in
 	if(Item.m_Time > 0)
 		pParam->m_plDemos->push_back(Item);
 
+	if(time_get_microseconds() - pRealUser->m_pThis->m_RaceDemosLoadStartTime > 500000)
+	{
+		pRealUser->m_pThis->GameClient()->m_Menus.RenderLoading(false, false);
+	}
+
 	return 0;
 }
 
-bool CRaceDemo::CheckDemo(int Time) const
+bool CRaceDemo::CheckDemo(int Time)
 {
 	std::vector<CDemoItem> lDemos;
 	CDemoListParam Param = {this, &lDemos, Client()->GetCurrentMap()};
-	Storage()->ListDirectoryInfo(IStorage::TYPE_SAVE, ms_pRaceDemoDir, RaceDemolistFetchCallback, &Param);
+	m_RaceDemosLoadStartTime = time_get_microseconds();
+	SRaceDemoFetchUser User;
+	User.m_pParam = &Param;
+	User.m_pThis = this;
+	Storage()->ListDirectoryInfo(IStorage::TYPE_SAVE, ms_pRaceDemoDir, RaceDemolistFetchCallback, &User);
 
 	// loop through demo files
 	for(auto &Demo : lDemos)
