@@ -336,7 +336,6 @@ class CEditorMap
 public:
 	CEditor *m_pEditor;
 	bool m_Modified;
-	int m_UndoModified;
 
 	CEditorMap()
 	{
@@ -393,7 +392,6 @@ public:
 	CEnvelope *NewEnvelope(int Channels)
 	{
 		m_Modified = true;
-		m_UndoModified++;
 		CEnvelope *e = new CEnvelope(Channels);
 		m_lEnvelopes.add(e);
 		return e;
@@ -404,7 +402,6 @@ public:
 	CLayerGroup *NewGroup()
 	{
 		m_Modified = true;
-		m_UndoModified++;
 		CLayerGroup *g = new CLayerGroup;
 		g->m_pMap = this;
 		m_lGroups.add(g);
@@ -420,7 +417,6 @@ public:
 		if(Index0 == Index1)
 			return Index0;
 		m_Modified = true;
-		m_UndoModified++;
 		swap(m_lGroups[Index0], m_lGroups[Index1]);
 		return Index1;
 	}
@@ -430,7 +426,6 @@ public:
 		if(Index < 0 || Index >= m_lGroups.size())
 			return;
 		m_Modified = true;
-		m_UndoModified++;
 		delete m_lGroups[Index];
 		m_lGroups.remove_index(Index);
 	}
@@ -438,7 +433,6 @@ public:
 	void ModifyImageIndex(INDEX_MODIFY_FUNC pfnFunc)
 	{
 		m_Modified = true;
-		m_UndoModified++;
 		for(int i = 0; i < m_lGroups.size(); i++)
 			m_lGroups[i]->ModifyImageIndex(pfnFunc);
 	}
@@ -446,7 +440,6 @@ public:
 	void ModifyEnvelopeIndex(INDEX_MODIFY_FUNC pfnFunc)
 	{
 		m_Modified = true;
-		m_UndoModified++;
 		for(int i = 0; i < m_lGroups.size(); i++)
 			m_lGroups[i]->ModifyEnvelopeIndex(pfnFunc);
 	}
@@ -454,7 +447,6 @@ public:
 	void ModifySoundIndex(INDEX_MODIFY_FUNC pfnFunc)
 	{
 		m_Modified = true;
-		m_UndoModified++;
 		for(int i = 0; i < m_lGroups.size(); i++)
 			m_lGroups[i]->ModifySoundIndex(pfnFunc);
 	}
@@ -504,6 +496,14 @@ enum
 	PROPTYPE_AUTOMAPPER,
 };
 
+enum
+{
+	DIRECTION_LEFT = 1,
+	DIRECTION_RIGHT = 2,
+	DIRECTION_UP = 4,
+	DIRECTION_DOWN = 8,
+};
+
 typedef struct
 {
 	int x, y;
@@ -641,15 +641,27 @@ class CEditor : public IEditor
 	CUI m_UI;
 	CUIEx m_UIEx;
 
+	bool m_EditorWasUsedBefore = false;
+
+	IGraphics::CTextureHandle m_EntitiesTexture;
+
+	IGraphics::CTextureHandle m_FrontTexture;
+	IGraphics::CTextureHandle m_TeleTexture;
+	IGraphics::CTextureHandle m_SpeedupTexture;
+	IGraphics::CTextureHandle m_SwitchTexture;
+	IGraphics::CTextureHandle m_TuneTexture;
+
+	int GetTextureUsageFlag();
+
 public:
-	class IInput *Input() { return m_pInput; };
-	class IClient *Client() { return m_pClient; };
+	class IInput *Input() { return m_pInput; }
+	class IClient *Client() { return m_pClient; }
 	class CConfig *Config() { return m_pConfig; }
-	class IConsole *Console() { return m_pConsole; };
-	class IGraphics *Graphics() { return m_pGraphics; };
+	class IConsole *Console() { return m_pConsole; }
+	class IGraphics *Graphics() { return m_pGraphics; }
 	class ISound *Sound() { return m_pSound; }
-	class ITextRender *TextRender() { return m_pTextRender; };
-	class IStorage *Storage() { return m_pStorage; };
+	class ITextRender *TextRender() { return m_pTextRender; }
+	class IStorage *Storage() { return m_pStorage; }
 	CUI *UI() { return &m_UI; }
 	CUIEx *UIEx() { return &m_UIEx; }
 	CRenderTools *RenderTools() { return &m_RenderTools; }
@@ -724,14 +736,15 @@ public:
 		m_AnimateSpeed = 1;
 
 		m_ShowEnvelopeEditor = 0;
-		m_ShowUndo = 0;
-		m_UndoScrollValue = 0.0f;
 		m_ShowServerSettingsEditor = false;
 
 		m_ShowEnvelopePreview = 0;
 		m_SelectedQuadEnvelope = -1;
 		m_SelectedEnvelopePoint = -1;
 
+		m_QuadKnifeActive = false;
+		m_QuadKnifeCount = 0;
+
 		m_CommandBox = 0.0f;
 		m_aSettingsCommand[0] = 0;
 
@@ -760,24 +773,6 @@ public:
 	virtual void UpdateMentions() { m_Mentions++; }
 	virtual void ResetMentions() { m_Mentions = 0; }
 
-	int64_t m_LastUndoUpdateTime;
-	bool m_UndoRunning;
-	void CreateUndoStep();
-	static void CreateUndoStepThread(void *pUser);
-	int UndoStep();
-	struct CUndo
-	{
-		int m_FileNum;
-		int m_ButtonId;
-		char m_aName[128];
-		IGraphics::CTextureHandle m_PreviewImage;
-		bool m_PreviewImageIsLoaded;
-	};
-	array<CUndo> m_lUndoSteps;
-	bool m_Undo;
-	int m_ShowUndo;
-	float m_UndoScrollValue;
-
 	CLayerGroup *m_apSavedBrushes[10];
 
 	void FilelistPopulate(int StorageType);
@@ -798,6 +793,7 @@ public:
 	CLayerGroup *GetSelectedGroup() const;
 	CSoundSource *GetSelectedSource();
 	void SelectLayer(int LayerIndex, int GroupIndex = -1);
+	void AddSelectedLayer(int LayerIndex);
 	void SelectQuad(int Index);
 	void DeleteSelectedQuads();
 	bool IsQuadSelected(int Index) const;
@@ -933,10 +929,15 @@ public:
 	int m_SelectedSound;
 	int m_SelectedSource;
 
+	bool m_QuadKnifeActive;
+	int m_QuadKnifeCount;
+	vec2 m_aQuadKnifePoints[4];
+
 	IGraphics::CTextureHandle m_CheckerTexture;
 	IGraphics::CTextureHandle m_BackgroundTexture;
 	IGraphics::CTextureHandle m_CursorTexture;
-	IGraphics::CTextureHandle m_EntitiesTexture;
+
+	IGraphics::CTextureHandle GetEntitiesTexture();
 
 	CLayerGroup m_Brush;
 	CLayerTiles m_TilesetPicker;
@@ -1028,6 +1029,10 @@ public:
 	void DoQuadEnvPoint(const CQuad *pQuad, int QIndex, int pIndex);
 	void DoQuadPoint(CQuad *pQuad, int QuadIndex, int v);
 
+	float TriangleArea(vec2 A, vec2 B, vec2 C);
+	bool IsInTriangle(vec2 Point, vec2 A, vec2 B, vec2 C);
+	void DoQuadKnife(int QuadIndex);
+
 	void DoSoundSource(CSoundSource *pSource, int Index);
 
 	void DoMapEditor(CUIRect View);
@@ -1048,7 +1053,6 @@ public:
 	void RenderModebar(CUIRect View);
 	void RenderStatusbar(CUIRect View);
 	void RenderEnvelopeEditor(CUIRect View);
-	void RenderUndoList(CUIRect View);
 	void RenderServerSettingsEditor(CUIRect View, bool ShowServerSettingsEditorLast);
 
 	void RenderMenubar(CUIRect Menubar);
@@ -1143,11 +1147,12 @@ public:
 
 	// DDRace
 
-	IGraphics::CTextureHandle m_FrontTexture;
-	IGraphics::CTextureHandle m_TeleTexture;
-	IGraphics::CTextureHandle m_SpeedupTexture;
-	IGraphics::CTextureHandle m_SwitchTexture;
-	IGraphics::CTextureHandle m_TuneTexture;
+	IGraphics::CTextureHandle GetFrontTexture();
+	IGraphics::CTextureHandle GetTeleTexture();
+	IGraphics::CTextureHandle GetSpeedupTexture();
+	IGraphics::CTextureHandle GetSwitchTexture();
+	IGraphics::CTextureHandle GetTuneTexture();
+
 	static int PopupTele(CEditor *pEditor, CUIRect View, void *pContext);
 	static int PopupSpeedup(CEditor *pEditor, CUIRect View, void *pContext);
 	static int PopupSwitch(CEditor *pEditor, CUIRect View, void *pContext);
