@@ -9,13 +9,7 @@
 #include "teams.h"
 #include <engine/shared/config.h>
 
-CSaveTee::CSaveTee()
-{
-}
-
-CSaveTee::~CSaveTee()
-{
-}
+CSaveTee::CSaveTee() = default;
 
 void CSaveTee::Save(CCharacter *pChr)
 {
@@ -436,7 +430,7 @@ CSaveTeam::~CSaveTeam()
 
 int CSaveTeam::Save(int Team)
 {
-	if(g_Config.m_SvTeam == 3 || (Team > 0 && Team < MAX_CLIENTS))
+	if(g_Config.m_SvTeam == SV_TEAM_FORCED_SOLO || (Team > 0 && Team < MAX_CLIENTS))
 	{
 		CGameTeams *Teams = &(((CGameControllerDDRace *)m_pController)->m_Teams);
 
