@@ -418,12 +418,12 @@ bool CScoreWorker::SaveScore(IDbConnection *pSqlServer, const ISqlData *pGameDat
 		}
 		pSqlServer->BindString(1, pData->m_aMap);
 
-		bool End;
-		if(pSqlServer->Step(&End, pError, ErrorSize))
+		bool End2;
+		if(pSqlServer->Step(&End2, pError, ErrorSize))
 		{
 			return true;
 		}
-		if(!End)
+		if(!End2)
 		{
 			int Points = pSqlServer->GetInt(1);
 			if(pSqlServer->AddPoints(pData->m_aName, Points, pError, ErrorSize))
@@ -482,7 +482,7 @@ bool CScoreWorker::SaveTeamScore(IDbConnection *pSqlServer, const ISqlData *pGam
 	// get the names sorted in a tab separated string
 	std::vector<std::string> aNames;
 	for(unsigned int i = 0; i < pData->m_Size; i++)
-		aNames.push_back(pData->m_aaNames[i]);
+		aNames.emplace_back(pData->m_aaNames[i]);
 
 	std::sort(aNames.begin(), aNames.end());
 	str_format(aBuf, sizeof(aBuf),
