@@ -141,6 +141,25 @@ TEST(Str, Startswith)
 	EXPECT_EQ(str_startswith(ABCDEFG, ABC) - ABCDEFG, str_length(ABC));
 }
 
+TEST(Str, StartswithNocase)
+{
+	EXPECT_TRUE(str_startswith_nocase("Abcdef", "abc"));
+	EXPECT_FALSE(str_startswith_nocase("aBc", "abcdef"));
+
+	EXPECT_TRUE(str_startswith_nocase("xYz", ""));
+	EXPECT_FALSE(str_startswith_nocase("", "xYz"));
+
+	EXPECT_FALSE(str_startswith_nocase("house", "home"));
+	EXPECT_FALSE(str_startswith_nocase("Blackboard", "board"));
+
+	EXPECT_TRUE(str_startswith_nocase("поплавать", "по"));
+	EXPECT_FALSE(str_startswith_nocase("плавать", "по"));
+
+	static const char ABCDEFG[] = "aBcdefg";
+	static const char ABC[] = "abc";
+	EXPECT_EQ(str_startswith_nocase(ABCDEFG, ABC) - ABCDEFG, str_length(ABC));
+}
+
 TEST(Str, Endswith)
 {
 	EXPECT_TRUE(str_endswith("abcdef", "def"));
@@ -161,6 +180,26 @@ TEST(Str, Endswith)
 		str_length(ABCDEFG) - str_length(DEFG));
 }
 
+TEST(Str, EndswithNocase)
+{
+	EXPECT_TRUE(str_endswith_nocase("abcdef", "deF"));
+	EXPECT_FALSE(str_endswith_nocase("def", "abcdef"));
+
+	EXPECT_TRUE(str_endswith_nocase("xyz", ""));
+	EXPECT_FALSE(str_endswith_nocase("", "xyz"));
+
+	EXPECT_FALSE(str_endswith_nocase("rhyme", "minE"));
+	EXPECT_FALSE(str_endswith_nocase("blackboard", "black"));
+
+	EXPECT_TRUE(str_endswith_nocase("люди", "юди"));
+	EXPECT_FALSE(str_endswith_nocase("люди", "любовь"));
+
+	static const char ABCDEFG[] = "abcdefG";
+	static const char DEFG[] = "defg";
+	EXPECT_EQ(str_endswith_nocase(ABCDEFG, DEFG) - ABCDEFG,
+		str_length(ABCDEFG) - str_length(DEFG));
+}
+
 TEST(Str, HexDecode)
 {
 	char aOut[5] = {'a', 'b', 'c', 'd', 0};
