@@ -47,13 +47,10 @@ void Run(unsigned short Port, NETADDR Dest)
 	NETADDR Src = {NETTYPE_IPV4, {0, 0, 0, 0}, Port};
 	NETSOCKET Socket = net_udp_create(Src);
 
-	char aBuffer[1024 * 2];
 	int ID = 0;
 	int Delaycounter = 0;
-	MMSGS m;
-	net_init_mmsgs(&m);
 
-	while(1)
+	while(true)
 	{
 		static int Lastcfg = 0;
 		int n = ((time_get() / time_freq()) / m_ConfigInterval) % m_ConfigNumpingconfs;
@@ -64,13 +61,13 @@ void Run(unsigned short Port, NETADDR Dest)
 		Lastcfg = n;
 
 		// handle incoming packets
-		while(1)
+		while(true)
 		{
 			// fetch data
 			int DataTrash = 0;
 			NETADDR From;
 			unsigned char *pData;
-			int Bytes = net_udp_recv(Socket, &From, aBuffer, 1024 * 2, &m, &pData);
+			int Bytes = net_udp_recv(Socket, &From, &pData);
 			if(Bytes <= 0)
 				break;
 
@@ -142,7 +139,7 @@ void Run(unsigned short Port, NETADDR Dest)
 		{*/
 		CPacket *p = 0;
 		CPacket *pNext = m_pFirst;
-		while(1)
+		while(true)
 		{
 			p = pNext;
 			if(!p)
@@ -208,7 +205,7 @@ void Run(unsigned short Port, NETADDR Dest)
 	}
 }
 
-int main(int argc, const char **argv) // ignore_convention
+int main(int argc, const char **argv)
 {
 	cmdline_fix(&argc, &argv);
 	NETADDR Addr = {NETTYPE_IPV4, {127, 0, 0, 1}, 8303};
