@@ -1,10 +1,10 @@
 /* (c) Magnus Auvinen. See licence.txt in the root of the distribution for more information. */
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #include <base/system.h>
+#include <cstdlib> //rand
 #include <engine/shared/config.h>
 #include <engine/shared/network.h>
 #include <mastersrv/mastersrv.h>
-#include <stdlib.h> //rand
 
 CNetServer *pNet;
 
@@ -113,10 +113,10 @@ static int Run()
 	int64_t NextHeartBeat = 0;
 	NETADDR BindAddr = {NETTYPE_IPV4, {0}, 0};
 
-	if(!pNet->Open(BindAddr, 0, 0, 0, 0))
+	if(!pNet->Open(BindAddr, 0, 0, 0))
 		return 0;
 
-	while(1)
+	while(true)
 	{
 		CNetChunk p;
 		pNet->Update();
